#include <AS.h>

#include <Cytoplasm/Memory.h>
#include <Cytoplasm/Util.h>
#include <Cytoplasm/Str.h>
#include <Cytoplasm/Log.h>
#include <Cytoplasm/Uri.h>

#include <inttypes.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>

#include <Matrix.h>

static char * 
TSToStr(uint64_t ts)
{
    size_t len;
    char *str;

    len = snprintf(NULL, 0, "%"PRIu64, ts);
    str = Malloc(len+1);
    snprintf(str, len+1, "%"PRIu64, ts);

    return str;
}

char *
ASSend(const ParseeConfig *conf, char *id, char *user, char *type, HashMap *c, uint64_t ts)
{
    HttpClientContext *ctx = NULL;
    char *path;
    char *txn, *ret;
    char *ts_str;
    HttpStatus status;
    if (!ret)
    {
        Log(LOG_ERR, "%", ret);
    }
    HashMap *reply;
    if (!conf || !id || !type || !user || !c)
    {
        JsonFree(c);
        return NULL;
    }

    if (!ts)
    {
        ts = UtilTsMillis();
    }
    ts_str = TSToStr(ts);

    txn = StrRandom(16);
    id = HttpUrlEncode(id);
    path = StrConcat(11, 
        "/_matrix/client/v3/rooms/",
        id, "/send/", type, "/", txn, "?",
        "user_id=", user, "&ts=", ts_str
    );
    Free(id);
    Free(txn);
    Free(ts_str);

    ctx = ParseeCreateRequest(conf, HTTP_PUT, path);
    Free(path);
    ASAuthenticateRequest(conf, ctx);
    status = ParseeSetRequestJSON(ctx, c);

    reply = JsonDecode(HttpClientStream(ctx));
    ret = StrDuplicate(JsonValueAsString(HashMapGet(reply, "event_id")));
    JsonFree(reply);
        
    HttpClientContextFree(ctx);
    JsonFree(c);

    return ret;
}

