#include <Command.h>

#include <Cytoplasm/Memory.h>
#include <Cytoplasm/Str.h>
#include <string.h>
#include <ctype.h>

typedef enum CommandState {
    STATE_WHITE,
    STATE_NAME,
    STATE_VALUE
} CommandState;

Command *
CommandParse(char *cmd)
{
    Command *ret;
    char *end_data, *cur, *namestart = NULL;
    char *key = NULL, *val = NULL;
    size_t len;
    CommandState state = STATE_WHITE;
    if (!cmd)
    {
        return NULL;
    }

    end_data = strchr(cmd, ' ');
    if (!end_data || (cmd > end_data))
    {
        ret = Malloc(sizeof(*ret));
        ret->command = StrDuplicate(cmd);
        ret->arguments = HashMapCreate();

        return ret;
    }

    len = end_data - cmd;
    ret = Malloc(sizeof(*ret));
    ret->arguments = HashMapCreate();
    ret->command = Malloc(len + 1);
    memcpy(ret->command, cmd, len);
    ret->command[len] = '\0';
    
    cur = end_data + 1;

    /* Parse arguments */
    while (*cur)
    {
        char c = *cur;
        char *tmp;
        bool type;
        char char_type = '\0';
        switch (state)
        {
            case STATE_WHITE:
                if (!isblank((int) c))
                {
                    state = STATE_NAME;
                    namestart = cur;
                    continue;
                }
                break;
            case STATE_NAME:
                if (c == '=' && namestart)
                {
                    len = cur - namestart;
                    key = Malloc(len + 1);
                    memcpy(key, namestart, len);
                    key[len] = '\0';
                    state = STATE_VALUE;
                    cur++;
                    val = NULL;
                    continue;
                }
                break;
            case STATE_VALUE:
                type = c == '"' || c == '\'';
                if (type)
                {
                    char_type = c;
                    cur++;
                }
                while (*cur)
                {
                    char c = *cur;
                    char cb[2] = { c, '\0' };
                    if ((type && c == char_type) || (!type && isblank((int) c)))
                    {
                        break;
                    }

                    if (c == '\\' && *(cur + 1) == char_type)
                    {
                        cb[0] = char_type;
                        cur++;
                    }
                    tmp = val;
                    val = StrConcat(2, val, cb);
                    Free(tmp);
                    
                    cur++;
                }
                Free(HashMapSet(ret->arguments, key, val));
                Free(key);
                key = NULL;
                val = NULL;
                state = STATE_WHITE;
                break;
        }
        cur++;
    }
    Free(key);
    Free(val);

    return ret;
}

void
CommandFree(Command *command)
{
    char *key, *val;
    if (!command)
    {
        return;
    }

    while (HashMapIterate(command->arguments, &key, (void **) &val))
    {
        Free(val);
    }
    HashMapFree(command->arguments);

    Free(command->command);
    Free(command);
}
