/* CC0 implementation of a HMAC system based on Cytoplasm.
 * Ignore the scary "Parsee.h", its practically unused. */
#include <Parsee.h>

#include <Cytoplasm/Memory.h>
#include <Cytoplasm/Sha.h>
#include <Cytoplasm/Str.h>

#include <string.h>

static void
ComputeKPad(char *key, uint8_t pad, uint8_t *kopad)
{
    size_t klen;
    uint8_t *kp;
    uint8_t kpi[64] = { 0 };
    size_t i;
    if ((klen = strlen(key)) <= 64)
    {
        kp = Malloc(klen * sizeof(uint8_t));
        memcpy(kp, key, klen);
    }
    else
    {
        kp = (uint8_t *) Sha256(key);
        klen = 32;
    }

    memset(kpi, 0x00, 64);
    memcpy(kpi, kp, klen);
    Free(kp);

    /* Now that we have K', lets compute it XORd with opad */
    for (i = 0; i < 64; i++)
    {
        uint8_t byte = kpi[i];
        kopad[i] = byte ^ pad;
    }
}

char *
ParseeHMAC(char *key, uint8_t *msg, size_t msglen)
{
    uint8_t opad[64], ipad[64 + msglen];
    uint8_t outer[64 + 32];

    uint8_t *innersha;
    uint8_t *sha;
    char *str;
    if (!key || !msg || !msglen)
    {
        return NULL;
    }

    /* Initialise K' XOR opad and K' XOR ipad */
    ComputeKPad(key, 0x5C, opad);
    ComputeKPad(key, 0x36, ipad);

    /* Compute H((K' XOR ipad) || msg) */
    memcpy(ipad + 64, msg, msglen);
    innersha = Sha256Raw(ipad, 64 + msglen);

    /* Compute (K' XOR opad) || H((K' XOR ipad) || msg) */
    memcpy(outer, opad, 64);
    memcpy(outer + 64, innersha, 32);

    /* Compute H((K' XOR opad) || H((K' XOR ipad) || msg)) */
    sha = Sha256Raw(outer, 64 + 32);
    str = ShaToHex(sha, HASH_SHA256);

    Free(innersha);
    Free(sha);
    return str;
}
