#include <Parsee.h>

#include <Cytoplasm/HashMap.h>
#include <Cytoplasm/Memory.h>
#include <Cytoplasm/Str.h>
#include <pthread.h>

static pthread_mutex_t affi_lock;
static HashMap *affi_table = NULL;

typedef struct XMPPStatus {
    char *role;
    char *affiliation;
} XMPPStatus;
static XMPPStatus *
CreateStatus(char *role, char *affiliation)
{
    XMPPStatus *ret;
    if (!role || !affiliation)
    {
        return NULL;
    }

    ret = Malloc(sizeof(*ret));
    ret->role = StrDuplicate(role);
    ret->affiliation = StrDuplicate(affiliation);

    return ret;
}
static void
FreeStatus(XMPPStatus *status)
{
    if (!status)
    {
        return;
    }

    Free(status->affiliation);
    Free(status->role);
    Free(status);
}

void
ParseeInitialiseAffiliationTable(void)
{
    if (affi_table)
    {
        return;
    }
    pthread_mutex_init(&affi_lock, NULL);
    pthread_mutex_lock(&affi_lock);
    affi_table = HashMapCreate();
    pthread_mutex_unlock(&affi_lock);
}
void
ParseePushAffiliationTable(char *user, char *affi, char *role)
{
    XMPPStatus *status;
    if (!user || !affi || !role)
    {
        return;
    }
    pthread_mutex_lock(&affi_lock);

    status = CreateStatus(role, affi);
    FreeStatus(HashMapSet(affi_table, user, status));

    pthread_mutex_unlock(&affi_lock);
}
bool
ParseeLookupAffiliation(char *user, char **affiliation, char **role)
{
    XMPPStatus *status;
    if (!user || !affiliation || !role)
    {
        return false;
    }
    pthread_mutex_lock(&affi_lock);

    status  = HashMapGet(affi_table, user);
    *affiliation = StrDuplicate(status ? status->affiliation : NULL);
    *role = StrDuplicate(status ? status->role : NULL);

    pthread_mutex_unlock(&affi_lock);
    return !!status;
}
void
ParseeDestroyAffiliationTable(void)
{
    char *key;
    void *val;
    if (!affi_table)
    {
        return;
    }
    pthread_mutex_lock(&affi_lock);
    while (HashMapIterate(affi_table, &key, &val))
    {
        FreeStatus(val);
    }
    HashMapFree(affi_table);
    affi_table = NULL;
    pthread_mutex_unlock(&affi_lock);
    pthread_mutex_destroy(&affi_lock);
}

