#include <Parsee.h>

#include <Cytoplasm/HashMap.h>
#include <Cytoplasm/Memory.h>
#include <Cytoplasm/Str.h>
#include <pthread.h>

static pthread_mutex_t head_lock;
static HashMap *head_table = NULL;
void
ParseeInitialiseHeadTable(void)
{
    if (head_table)
    {
        return;
    }
    pthread_mutex_init(&head_lock, NULL);
    pthread_mutex_lock(&head_lock);
    head_table = HashMapCreate();
    pthread_mutex_unlock(&head_lock);
}
void
ParseePushHeadTable(char *room, char *event)
{
    if (!room || !event || !head_table)
    {
        return;
    }
    pthread_mutex_lock(&head_lock);
    event = StrDuplicate(event);
    Free(HashMapSet(head_table, room, event));
    pthread_mutex_unlock(&head_lock);
}
char *
ParseeLookupHead(char *room)
{
    char *event;
    if (!room || !head_table)
    {
        return NULL;
    }
    pthread_mutex_lock(&head_lock);
    event = StrDuplicate(HashMapGet(head_table, room));
    pthread_mutex_unlock(&head_lock);

    return event;
}
void
ParseeDestroyHeadTable(void)
{
    char *key;
    void *val;
    if (!head_table)
    {
        return;
    }
    pthread_mutex_lock(&head_lock);
    while (HashMapIterate(head_table, &key, &val))
    {
        Free(val);
    }
    HashMapFree(head_table);
    head_table = NULL;
    pthread_mutex_unlock(&head_lock);
    pthread_mutex_destroy(&head_lock);
}

