#include <Parsee.h>

#include <Cytoplasm/HashMap.h>
#include <Cytoplasm/Memory.h>
#include <Cytoplasm/Str.h>
#include <pthread.h>

static pthread_mutex_t lock;
static HashMap *jid_table = NULL;

void
ParseeInitialiseJIDTable(void)
{
    if (jid_table)
    {
        return;
    }
    pthread_mutex_init(&lock, NULL);
    pthread_mutex_lock(&lock);
    jid_table = HashMapCreate();
    pthread_mutex_unlock(&lock);
}
void
ParseePushJIDTable(char *muc, char *bare)
{
    if (!muc || !bare || !jid_table)
    {
        return;
    }
    pthread_mutex_lock(&lock);
    bare = ParseeTrimJID(bare);
    Free(HashMapSet(jid_table, muc, bare));
    pthread_mutex_unlock(&lock);
}
char *
ParseeLookupJID(char *muc)
{
    char *bare;
    if (!muc || !jid_table)
    {
        return NULL;
    }
    pthread_mutex_lock(&lock);
    bare = StrDuplicate(HashMapGet(jid_table, muc));
    pthread_mutex_unlock(&lock);

    if (!bare)
    {
        bare = StrDuplicate(muc);
    }
    return bare;
}
void
ParseeDestroyJIDTable(void)
{
    char *key;
    void *val;
    if (!jid_table)
    {
        return;
    }
    pthread_mutex_lock(&lock);
    while (HashMapIterate(jid_table, &key, &val))
    {
        Free(val);
    }
    HashMapFree(jid_table);
    jid_table = NULL;
    pthread_mutex_unlock(&lock);
    pthread_mutex_destroy(&lock);
}

