#include <Parsee.h>

#include <Cytoplasm/HashMap.h>
#include <Cytoplasm/Memory.h>
#include <Cytoplasm/Str.h>
#include <pthread.h>

static pthread_mutex_t nick_lock;
static HashMap *nick_table = NULL;

static char *
GenerateKey(char *muc, char *mxid)
{
    char *concatStr;
    char *hexDigest;
    if (!muc || !mxid)
    {
        return NULL;
    }

    concatStr = StrConcat(3, muc, ":", mxid);
    hexDigest = ParseeSHA256(concatStr);

    Free (concatStr);
    return hexDigest;
}

void
ParseeInitialiseNickTable(void)
{
    if (nick_table)
    {
        return;
    }
    pthread_mutex_init(&nick_lock, NULL);
    pthread_mutex_lock(&nick_lock);
    nick_table = HashMapCreate();
    pthread_mutex_unlock(&nick_lock);
}
void
ParseePushNickTable(char *muc, char *mxid, char *nick)
{
    char *key;
    if (!muc || !mxid || !nick_table)
    {
        return;
    }
    pthread_mutex_lock(&nick_lock);

    key = GenerateKey(muc, mxid);
    nick = StrDuplicate(nick);
    if (nick)
    {
        Free(HashMapSet(nick_table, key, nick));
    }
    else
    {
        Free(HashMapDelete(nick_table, key));
    }
    Free(key);

    pthread_mutex_unlock(&nick_lock);
}
char *
ParseeLookupNick(char *muc, char *mxid)
{
    char *ret, *key;
    if (!muc || !nick_table)
    {
        return NULL;
    }
    pthread_mutex_lock(&nick_lock);

    key = GenerateKey(muc, mxid);
    ret = HashMapGet(nick_table, key);
    Free(key);

    pthread_mutex_unlock(&nick_lock);
    return ret;
}
void
ParseeDestroyNickTable(void)
{
    char *key;
    void *val;
    if (!nick_table)
    {
        return;
    }
    pthread_mutex_lock(&nick_lock);
    while (HashMapIterate(nick_table, &key, &val))
    {
        Free(val);
    }
    HashMapFree(nick_table);
    nick_table = NULL;
    pthread_mutex_unlock(&nick_lock);
    pthread_mutex_destroy(&nick_lock);
}

