#include <Parsee.h>

#include <Cytoplasm/HashMap.h>
#include <Cytoplasm/Memory.h>
#include <Cytoplasm/Str.h>
#include <pthread.h>

static pthread_mutex_t oid_lock;
static HashMap *oid_table = NULL;

void
ParseeInitialiseOIDTable(void)
{
    if (oid_table)
    {
        return;
    }
    pthread_mutex_init(&oid_lock, NULL);
    pthread_mutex_lock(&oid_lock);
    oid_table = HashMapCreate();
    pthread_mutex_unlock(&oid_lock);
}
void
ParseePushOIDTable(char *muc, char *bare)
{
    if (!muc || !bare || !oid_table)
    {
        return;
    }
    pthread_mutex_lock(&oid_lock);
    bare = StrDuplicate(bare);
    Free(HashMapSet(oid_table, muc, bare));
    pthread_mutex_unlock(&oid_lock);
}
char *
ParseeLookupOID(char *muc)
{
    char *bare;
    if (!muc || !oid_table)
    {
        return NULL;
    }
    pthread_mutex_lock(&oid_lock);
    bare = StrDuplicate(HashMapGet(oid_table, muc));
    pthread_mutex_unlock(&oid_lock);

    return bare;
}
void
ParseeDestroyOIDTable(void)
{
    char *key;
    void *val;
    if (!oid_table)
    {
        return;
    }
    pthread_mutex_lock(&oid_lock);
    while (HashMapIterate(oid_table, &key, &val))
    {
        Free(val);
    }
    HashMapFree(oid_table);
    oid_table = NULL;
    pthread_mutex_unlock(&oid_lock);
    pthread_mutex_destroy(&oid_lock);
}

