#include <Parsee.h>

#include <Cytoplasm/Memory.h>
#include <Cytoplasm/Json.h>
#include <Cytoplasm/Util.h>
#include <Cytoplasm/Str.h>
#include <Cytoplasm/Log.h>

#include <string.h>
#include <stdlib.h>
#include <ctype.h>

#include <Matrix.h>

bool
ParseeIsPuppet(const ParseeConfig *conf, char *user)
{
    UserID *id;
    bool flag = true;
    size_t len;
    if (!user           || 
        !conf           || 
        *user != '@'    ||
        !(id = MatrixParseID(user)))
    {
        return false;
    }

    len = strlen(conf->namespace_base);
    if (strncmp(id->localpart, conf->namespace_base, len))
    {
        flag = false;
    }

    if (StrEquals(id->localpart, conf->sender_localpart))
    {
        flag = true;
    }

    flag = flag && StrEquals(id->server, conf->server_base);

    Free(id);
    return flag;
}
char *
ParseeDecodeJID(char *str, char term)
{
    char *out = NULL;
    if (!str)
    {
        return NULL;
    }
#define Okay(c) ((c) && ((c) != term))
    while (Okay(*str))
    {
        char c = *str;
        char buf[3] = { 0 };
        char *tmp;
        if (c == '=' && Okay(*(str + 1)) && Okay(*(str + 2)))
        {
            str++;

            memcpy(buf, str, 2);
            buf[0] = strtol(buf, NULL, 16);
            buf[1] = '\0';

            tmp = StrConcat(2, out, buf);
            Free(out);
            out = tmp;

            str += 2;
            continue;
        }
        memcpy(buf, str, 1);
        tmp = StrConcat(2, out, buf);
        Free(out);
        out = tmp;
        
        str++;
    }
#undef Okay
    return out;
}
char *
ParseeDecodeLocalJID(const ParseeConfig *c, char *mxid)
{
    char *localpart, *jid_flags, *data_start;
    bool plain_jid = false;
    if (!ParseeIsPuppet(c, mxid))
    {
        return NULL;
    }

    localpart = mxid + 1;
    jid_flags = localpart + strlen(c->namespace_base) + 1;
    data_start = jid_flags;
    while (*data_start && *data_start != '_')
    {
        /* TODO: Get rid of this */
        if (*data_start == 'l')
        {
            plain_jid = true;
        }
        data_start++;
    }
    if (!*data_start || !plain_jid)
    {
        return NULL;
    }
    data_start++;
    /* Until the ':', data_start now is an encoded JID */
    return ParseeDecodeJID(data_start, ':');
}
char *
ParseeDecodeLocalMUC(const ParseeConfig *c, char *alias)
{
    char *localpart, *jid_flags, *data_start;
    bool plain_jid = false;
    if (!c || !alias || *alias != '#')
    {
        return NULL;
    }

    localpart = alias + 1;
    jid_flags = localpart + strlen(c->namespace_base) + 1;
    data_start = jid_flags;
    while (*data_start && *data_start != '_')
    {
        /* TODO: Get rid of this */
        if (*data_start == 'm')
        {
            plain_jid = true;
        }
        data_start++;
    }
    if (!*data_start || !plain_jid)
    {
        return NULL;
    }
    data_start++;
    /* Until the ':', data_start now is an encoded JID */
    return ParseeDecodeJID(data_start, ':');
}

static const char *HEX = "0123456789abcdef";
char *
ParseeEncodeJID(const ParseeConfig *c, char *jid, bool trim)
{
    char *ret, *tmp;
    size_t i, len;
    if (!c || !jid)
    {
        return NULL;
    }

    ret = StrConcat(2, c->namespace_base, "_l_");
    len = strlen(jid);
    for (i = 0; i < len; i++)
    {
        char cpy = jid[i];
        char cs[4] = { 0 };
        cs[0] = cpy;
        if (*cs == '/' && trim)
        {
            /* RID: Break everything and die. */
            break;
        }
        if (islower((int) *cs) || isalnum((int) *cs) || *cs == '_' ||
            *cs == '=' || *cs == '-' || *cs == '/' || 
            *cs == '+' || *cs == '.')
        {
            tmp = ret;
            ret = StrConcat(2, ret, cs);
            Free(tmp);
            continue;
        }

        /* Hex encode everything */
        cs[0] = '=';
        cs[1] = HEX[(cpy >> 4) & 0xF];
        cs[2] = HEX[(cpy >> 0) & 0xF];
        tmp = ret;
        ret = StrConcat(2, ret, cs);
        Free(tmp);
    }
    tmp = ret;
    ret = StrConcat(4, "@", ret, ":", c->server_base);
    Free(tmp);

    return ret;
}

char *
ParseeGetLocal(char *mxid)
{
    char *cpy;
    size_t i, len;
    if (!mxid)
    {
        return NULL;
    }
    if (*mxid != '@')
    {
        return StrDuplicate(mxid);
    }

    len = strlen(mxid);

    mxid++;
    cpy = Malloc(len + 1);
    memset(cpy, '\0', len + 1);
    memcpy(cpy, mxid, len);

    for (i = 0; i < len; i++)
    {
        if (cpy[i] == ':')
        {
            cpy[i] = '\0';
            break;
        }
    }

    return cpy;
}

char *
ParseeEncodeMXID(char *mxid)
{
    char *ret;
    size_t i, j, len;
    if (!mxid)
    {
        return NULL;
    }

    /* Worst case scenario of 3-bytes the char */
    len = strlen(mxid);
    ret = Malloc(len * 3 + 1);
    for (i = 0, j = 0; i < len; i++)
    {
        char src = mxid[i];

        if (src <= 0x20 ||
            (src == '"' || src == '&' ||
              src == '\'' || src == '/' ||
              src == ':' || src == '<' || 
              src == '>' || src == '@' ||
              src == 0x7F))
        {
            ret[j++] = '?';
            ret[j++] = HEX[(src & 0xF0) >> 4];
            ret[j++] = HEX[(src & 0x0F) >> 0];
            continue;
        }
        ret[j++] = src;
    }
    ret[j] = '\0';

    return ret;
}
char *
ParseeDecodeMXID(char *mxid)
{
    char *out = NULL;
    if (!mxid)
    {
        return NULL;
    }
#define Okay(c) ((c) && (c != '@'))
    while (Okay(*mxid))
    {
        char c = *mxid;
        char buf[3] = { 0 };
        char *tmp;
        if (c == '?' && Okay(*(mxid + 1)) && Okay(*(mxid + 2)))
        {
            mxid++;

            memcpy(buf, mxid, 2);
            buf[0] = strtol(buf, NULL, 16);
            buf[1] = '\0';

            tmp = StrConcat(2, out, buf);
            Free(out);
            out = tmp;

            mxid += 2;
            continue;
        }
        memcpy(buf, mxid, 1);
        tmp = StrConcat(2, out, buf);
        Free(out);
        out = tmp;
        
        mxid++;
    }
#undef Okay
    return out;
}
char *
ParseeGetDMID(char *mxid, char *jid)
{
    char *concat, *sha;
    if (!mxid || !jid)
    {
        return NULL;
    }

    /* We really don't care about safety here. */
    jid = ParseeTrimJID(jid);
    concat = StrConcat(2, mxid, jid);
    sha = ParseeSHA1(concat);

    Free(concat);
    Free(jid);

    return sha;
}
char *
ParseeFindDMRoom(ParseeData *data, char *mxid, char *jid)
{
    DbRef *ref;
    HashMap *j;
    char *room, *dmid;
    if (!data || !mxid || !jid)
    {
        return NULL;
    }
    dmid = ParseeGetDMID(mxid, jid);
    ref = DbLockIntent(data->db, DB_HINT_READONLY, 2, "users", dmid);
    j   = DbJson(ref);

    room = StrDuplicate(JsonValueAsString(HashMapGet(j, "room")));

    DbUnlock(data->db, ref);
    Free(dmid);
    return room;
}

void
ParseePushDMRoom(ParseeData *d, char *mxid, char *jid, char *r)
{
    DbRef *ref;
    HashMap *j;
    char *dmid;
    if (!d || !mxid || !jid || !r)
    {
        return;
    }
    dmid = ParseeGetDMID(mxid, jid);
    ref = DbCreate(d->db, 2, "users", dmid);
    j   = DbJson(ref);

    HashMapSet(j, "room", JsonValueString(r));

    DbUnlock(d->db, ref);
    Free(dmid);
    return;
}
void
ParseeDeleteDM(ParseeData *d, char *mxid, char *jid)
{
    char *dmid;
    if (!d || !mxid || !jid)
    {
        return;
    }

    dmid = ParseeGetDMID(mxid, jid);
    DbDelete(d->db, 2, "users", dmid);
    Free(dmid);
    return;
}

char *
ParseeTrimJID(char *jid)
{
    char *ret;
    size_t i, len;
    if (!jid)
    {
        return NULL;
    }

    ret = StrDuplicate(jid);
    len = strlen(ret);
    for (i = 0; i < len; i++)
    {
        if (ret[i] == '/')
        {
            ret[i] = '\0';
            break;
        }
    }

    return ret;
}
char *
ParseeGetResource(char *jid)
{
    if (!jid)
    {
        return NULL;
    }
    while (*jid && *jid != '/')
    {
        jid++;
    }
    if (!*jid)
    {
        return NULL;
    }

    return StrDuplicate(jid + 1);
}
char *
ParseePushMUC(ParseeData *data, char *room_id, char *jid)
{
    DbRef *ref;
    HashMap *j;
    char *chatid = NULL;
    if (!data || !room_id || !jid)
    {
        return NULL;
    }

    chatid = ParseeGetDMID(room_id, jid);
    ref = DbCreate(data->db, 2, "chats", chatid);
    j = DbJson(ref);

    HashMapSet(j, "room_id", JsonValueString(room_id));
    HashMapSet(j, "jabber_id", JsonValueString(jid));

    DbUnlock(data->db, ref);

    /* Create a double-mapping */
    ref = DbLockIntent(data->db, DB_HINT_WRITE, 1, "chats");
    if (!ref)
    {
        ref = DbCreate(data->db, 1, "chats");
        if (!ref)
        {
            Free(chatid);
            return NULL;
        }
    }
    j = DbJson(ref);
    JsonSet(j, JsonValueString(chatid), 2, "mucs", jid);
    JsonSet(j, JsonValueString(chatid), 2, "rooms", room_id);
    DbUnlock(data->db, ref);

    return chatid;
}

char *
ParseeGetFromMUCID(ParseeData *data, char *jid)
{
    DbRef *ref;
    HashMap *j;
    char *cid;
    if (!data || !jid)
    {
        return NULL;
    }

    jid = ParseeTrimJID(jid);

    ref = DbLockIntent(data->db, DB_HINT_READONLY, 1, "chats");
    j = DbJson(ref);

    cid = JsonValueAsString(JsonGet(j, 2, "mucs", jid));
    cid = StrDuplicate(cid);

    DbUnlock(data->db, ref);
    Free(jid);

    return cid;
}

char *
ParseeGetFromRoomID(ParseeData *data, char *room_id)
{
    DbRef *ref;
    HashMap *j;
    char *cid;
    if (!data || !room_id)
    {
        return NULL;
    }

    ref = DbLockIntent(data->db, DB_HINT_READONLY, 1, "chats");
    j = DbJson(ref);

    cid = JsonValueAsString(JsonGet(j, 2, "rooms", room_id));
    cid = StrDuplicate(cid);

    DbUnlock(data->db, ref);

    return cid;
}
char *
ParseeGetRoomID(ParseeData *data, char *chat_id)
{
    DbRef *ref;
    HashMap *j;
    char *ret;
    if (!data || !chat_id)
    {
        return NULL;
    }

    ref = DbLockIntent(data->db, DB_HINT_READONLY, 2, "chats", chat_id);
    j = DbJson(ref);

    ret = StrDuplicate(JsonValueAsString(HashMapGet(j, "room_id")));

    DbUnlock(data->db, ref);
    return ret;
}

char *
ParseeGetMUCID(ParseeData *data, char *chat_id)
{
    DbRef *ref;
    HashMap *j;
    char *ret;
    if (!data || !chat_id)
    {
        return NULL;
    }

    ref = DbLockIntent(data->db, DB_HINT_READONLY, 2, "chats", chat_id);
    j = DbJson(ref);

    ret = StrDuplicate(JsonValueAsString(HashMapGet(j, "jabber_id")));

    DbUnlock(data->db, ref);
    return ret;
}

void
ParseeSendPresence(ParseeData *data)
{
    DbRef *ref;
    HashMap *j, *mucs;
    JsonValue *val;
    char *muc;
    if (!data)
    {
        return;
    }
    ref = DbLockIntent(data->db, DB_HINT_READONLY, 1, "chats");
    j = DbJson(ref);
    mucs = JsonValueAsObject(HashMapGet(j, "mucs"));
    
    while (HashMapIterate(mucs, &muc, (void **) &val))
    {
        char *rev = StrConcat(2, muc, "/parsee");
        char *chat_id = ParseeGetFromMUCID(data, muc);
        DbRef *chat = DbLockIntent(data->db, DB_HINT_READONLY, 2, "chats", chat_id);
        uint64_t ts = GrabInteger(DbJson(chat), 1, "ts");
        int diff = ts ? (int) ((UtilTsMillis() - ts) / 1000) : -1;
        /* Make a fake user join the MUC */
        Log(LOG_NOTICE, "Sending presence to %s", rev);
        XMPPJoinMUC(data->jabber, "parsee", rev, NULL, diff, false);

        DbUnlock(data->db, chat);
        Free(chat_id);
        Free(rev);
    }
    DbUnlock(data->db, ref);
}
bool
ParseeGetDMStanzaInfo(ParseeData *data, char *room_id, char *ev, char **st, char **se)
{
    DbRef *ref = NULL;
    HashMap *j = NULL;
    HashMap *event = NULL;
    bool ret = false;
    if (!data || !room_id || !ev || !st || !se)
    {
        return false;
    }

    ref = DbLockIntent(data->db, 3, DB_HINT_READONLY, "rooms", room_id, "data");
    j = DbJson(ref);
    if (!ref)
    {
        goto end;
    }


    event = JsonValueAsObject(JsonGet(j, 2, "events", ev));
    if (!event)
    {
        goto end;
    }
    *st = StrDuplicate(JsonValueAsString(HashMapGet(event, "origin")));
    *se = StrDuplicate(JsonValueAsString(HashMapGet(event, "sender")));
    ret = true;
end:
    DbUnlock(data->db, ref);
    return ret;
}
bool
ParseeGetStanzaInfo(ParseeData *data, char *chat_id, char *ev, char **st, char **se)
{
    DbRef *ref = NULL;
    HashMap *j = NULL;
    bool ret = false;
    if (!data || !chat_id || !ev || !st || !se)
    {
        return false;
    }

    ref = DbLockIntent(data->db, DB_HINT_READONLY, 4, "chats", chat_id, "events", ev);
    j = DbJson(ref);
    if (!ref)
    {
        goto end;
    }

    *st = StrDuplicate(JsonValueAsString(HashMapGet(j, "stanza")));
    *se = StrDuplicate(JsonValueAsString(HashMapGet(j, "sender")));
    ret = true;
end:
    DbUnlock(data->db, ref);
    return ret;
}
bool
ParseeGetDMOrigin(ParseeData *data, char *room_id, char *ev, char **o)
{
    DbRef *ref = NULL;
    HashMap *j = NULL;
    HashMap *event = NULL;
    bool ret = false;
    if (!data || !room_id || !ev || !o)
    {
        return false;
    }

    ref = DbLockIntent(data->db, DB_HINT_READONLY, 3, "rooms", room_id, "data");
    j = DbJson(ref);
    if (!ref)
    {
        goto end;
    }


    event = JsonValueAsObject(JsonGet(j, 2, "events", ev));
    if (!event)
    {
        goto end;
    }
    *o = StrDuplicate(JsonValueAsString(HashMapGet(event, "origin")));
    ret = true;
end:
    DbUnlock(data->db, ref);
    return ret;

}
bool
ParseeGetOrigin(ParseeData *data, char *chat_id, char *ev, char **o)
{
    DbRef *ref = NULL;
    HashMap *j = NULL;
    bool ret = false;
    if (!data || !chat_id || !ev || !o)
    {
        return false;
    }

    ref = DbLockIntent(data->db, DB_HINT_READONLY, 4, "chats", chat_id, "events", ev);
    j = DbJson(ref);
    if (!ref)
    {
        goto end;
    }

    *o = StrDuplicate(JsonValueAsString(HashMapGet(j, "origin")));
    ret = true;
end:
    DbUnlock(data->db, ref);
    return ret;

}

#include <Cytoplasm/Uri.h>
char *
ParseeToUnauth(ParseeData *data, char *mxc, char *filename)
{
    Uri *url = NULL;
    char *ret;
    char *key, *hmac;
#define PAT "%s/media/%s%s?hmac=%s"
#define PATF "%s/media/%s%s/%s?hmac=%s"
    size_t l;
    if (!data || !mxc)
    {
        return NULL;
    }
    url = UriParse(mxc);
    if (!url)
    {
        return NULL;
    }
    if (!StrEquals(url->proto, "mxc"))
    {
        UriFree(url);
        return NULL;
    }

    key = StrConcat(2, url->host, url->path);
    hmac = ParseeHMACS(data->id, key);
    Free(key);

    if (!filename)
    {
        l = snprintf(NULL, 0, 
            PAT,
            data->config->media_base, 
            url->host, url->path,
            hmac
        );
    }
    else
    {
        l = snprintf(NULL, 0, 
            PATF,
            data->config->media_base, 
            url->host, url->path, filename,
            hmac
        );
    }
    ret = Malloc(l + 3);
    if (!filename)
    {
        snprintf(ret, l + 1, 
            PAT,
            data->config->media_base, 
            url->host, url->path,
            hmac
        );
    }
    else
    {
        snprintf(ret, l + 1, 
            PATF,
            data->config->media_base, 
            url->host, url->path, filename,
            hmac
        );
    }
    UriFree(url);
    Free(hmac);
    return ret;
}

#include <Glob.h>
bool
ParseeIsAdmin(ParseeData *data, char *user)
{
    DbRef *admins;
    HashMap *json;
    Array *values;
    size_t i;
    bool ret = false;
    if (!data || !user)
    {
        return false;
    }

    admins = DbLockIntent(data->db, DB_HINT_READONLY, 1, "admins");
    json = DbJson(admins);

    values = GrabArray(json, 1, "admins");
    for (i = 0; i < ArraySize(values); i++)
    {
        char *glob = JsonValueAsString(ArrayGet(values, i));
        if ((ret = GlobMatches(glob, user)))
        {
            break;
        }
    }

    DbUnlock(data->db, admins);
    return ret;
}

char *
ParseeJID(ParseeData *data)
{
    return StrConcat(2, "parsee@", data->config->component_host);
}
char *
ParseeMXID(ParseeData *data)
{
    return StrConcat(4, "@", data->config->sender_localpart,
                        ":", data->config->server_base);
}
