#include <Parsee.h>

#include <Cytoplasm/HashMap.h>
#include <Cytoplasm/Memory.h>
#include <Cytoplasm/Http.h>
#include <Cytoplasm/Json.h>
#include <Cytoplasm/Str.h>
#include <Cytoplasm/Uri.h>
#include <Cytoplasm/Log.h>

#include <stdbool.h>
#include <string.h>

#include <StringStream.h>
#include <Matrix.h>
#include <XML.h>
#include <AS.h>

typedef struct XMPPFlags {
    bool quote;
} XMPPFlags;
static char *
XMPPifyElement(const ParseeConfig *conf, HashMap *event, XMLElement *elem, XMPPFlags flags)
{
    char *xepd = NULL, *tmp = NULL;

    size_t i;
    XMLElement *child;
    char *reply_id = GrabString(
        event,
        4, "content", 
        "m.relates_to", "m.in_reply_to", "event_id"
    );
    char *room_id = JsonValueAsString(HashMapGet(event, "room_id"));
    HashMap *referenced;
    char *subxep;
#define Concat(strp)    do \
                        { \
                            size_t cidx; \
                            size_t len = strp ? strlen(strp) : 0; \
                            for (cidx = 0; cidx < len; cidx++) \
                            { \
                                char cch[2]; \
                                cch[0] = strp[cidx]; \
                                cch[1] = '\0'; \
                                char nch = *cch ? strp[cidx+1] : '\0'; \
                                bool c = *cch == '\n' && nch != '>'; \
                                if (c && flags.quote) \
                                { \
                                    tmp = xepd; \
                                    xepd = StrConcat(2, xepd, "\n>"); \
                                    Free(tmp); \
                                    continue; \
                                } \
                                tmp = xepd; \
                                xepd = StrConcat(2, xepd, cch); \
                                Free(tmp); \
                            } \
                        } \
                        while (0)
    switch (elem ? elem->type : -1)
    {
        case XML_ELEMENT_DATA:
            Concat(elem->data);
            break;
        case XML_ELEMENT_TAG:
            if (StrEquals(elem->name, "b") || StrEquals(elem->name, "strong"))
            {
                Concat("*");
                for (i = 0; i < ArraySize(elem->children); i++)
                {
                    child = ArrayGet(elem->children, i);
                    subxep = XMPPifyElement(conf, event, child, flags);

                    Concat(subxep);
                    Free(subxep);
                }
                Concat("*");
            }
            else if (StrEquals(elem->name, "em"))
            {
                Concat("_");
                for (i = 0; i < ArraySize(elem->children); i++)
                {
                    child = ArrayGet(elem->children, i);
                    subxep = XMPPifyElement(conf, event, child, flags);

                    Concat(subxep);
                    Free(subxep);
                }
                Concat("_");
            }
            else if (StrEquals(elem->name, "code"))
            {
                Concat("`");
                for (i = 0; i < ArraySize(elem->children); i++)
                {
                    child = ArrayGet(elem->children, i);
                    subxep = XMPPifyElement(conf, event, child, flags);

                    Concat(subxep);
                    Free(subxep);
                }
                Concat("`");
            }
            else if (StrEquals(elem->name, "mx-reply"))
            {
                char *str;
                referenced = ASFind(ParseeConfigGet(), room_id, reply_id);
                str = JsonValueAsString(
                    JsonGet(referenced, 2, "content", "body")
                );
                if (!str)
                {
                    JsonFree(referenced);
                    return xepd;
                }
                Concat(">");
                flags.quote = true;
                Concat(str);
                flags.quote = false;
                Concat("\n");
                JsonFree(referenced);
            }
            else if (StrEquals(elem->name, "blockquote"))
            {
                Concat(">");
                flags.quote = true;
                for (i = 0; i < ArraySize(elem->children); i++)
                {
                    child = ArrayGet(elem->children, i);
                    subxep = XMPPifyElement(conf, event, child, flags);

                    Concat(subxep);
                    Free(subxep);
                }
                flags.quote = false;
                Concat("\n");
            }
            else if (StrEquals(elem->name, "br"))
            {
                Concat("\n");
                /* HTML fucking SUCKS */
                for (i = 0; i < ArraySize(elem->children); i++)
                {
                    child = ArrayGet(elem->children, i);
                    subxep = XMPPifyElement(conf, event, child, flags);

                    Concat(subxep);
                    Free(subxep);
                }
                if (i != 0)
                {
                    Concat("\n");
                }
            }
            else if (StrEquals(elem->name, "a"))
            {
                char *href = HashMapGet(elem->attrs, "href");
                Uri *pref = UriParse(href);
                if (pref && StrEquals(pref->host, "matrix.to"))
                {
                    /* TODO: Check if the element here is a Matrix.TO 
                     * pointing to a Parsee user. */
                    UserID *id = MatrixParseIDFromMTO(pref);
                    if (id)
                    {
                        char *real_id = StrConcat(4, "@", id->localpart, ":", id->server);
                        /* TODO: Detect if it already is a Parsee user */
                        if (ParseeIsPuppet(conf, real_id))
                        {
                            char *name = ASGetName(conf, NULL, real_id);
                            Concat((name ? name : real_id));
                            Free(name);
                        }
                        else
                        {
                            Concat(real_id);
                        }
                        Free(real_id);
                    }
                    else
                    {
                        Concat(href);
                    }
                    
                    Free(id);
                }
                else
                {
                    for (i = 0; i < ArraySize(elem->children); i++)
                    {
                        child = ArrayGet(elem->children, i);
                        subxep = XMPPifyElement(conf, event, child, flags);

                        Concat(subxep);
                        Free(subxep);
                    }
                    Concat(" < ");
                    Concat(href);
                    Concat(" >");
                }
                UriFree(pref);
            }
            else
            {
                for (i = 0; i < ArraySize(elem->children); i++)
                {
                    child = ArrayGet(elem->children, i);
                    subxep = XMPPifyElement(conf, event, child, flags);

                    Concat(subxep);
                    Free(subxep);
                }
            }
            break;
        default:
            break;
    }
    return xepd;
}
static char *
GetRawBody(HashMap *event)
{
    void *id;
    if ((id = MatrixGetEdit(event)))
    {
        char *new = GrabString(event, 3, "content", "m.new_content", "body");
        Free(id);
        if (new)
        {
            return new;
        }
    }
    return GrabString(event, 2, "content", "body");
}
static char *
GetHTMLBody(HashMap *event)
{
    if (MatrixGetEdit(event))
    {
        char *new = GrabString(event, 3, "content", "m.new_content", "formatted_body");
        if (new)
        {
            return new;
        }
    }
    return GrabString(event, 2, "content", "formatted_body");
}
static char *
GetBodyFormat(HashMap *event)
{
    if (MatrixGetEdit(event))
    {
        return GrabString(event, 3, "content", "m.new_content", "format");
    }
    return GrabString(event, 2, "content", "format");
}
char *
ParseeXMPPify(ParseeData *data, HashMap *event)
{
    char *type, *format, *html;
    char *xepd = NULL;
    XMLElement *elem;

    XMPPFlags flags;
    if (!event)
    {
        return NULL;
    }

    /* Check if it is a message event. */
    type = JsonValueAsString(HashMapGet(event, "type"));
    if (!StrEquals(type, "m.room.message"))
    {
        return NULL;
    }

    if (!StrEquals(GetBodyFormat(event), "org.matrix.custom.html"))
    {
        /* Settle for the raw body instead. */
        char *body = GetRawBody(event);
        return StrDuplicate(body);
    }

    html = GetHTMLBody(event);
   
    html = StrConcat(3, "<html>", html, "</html>");
    elem = XMLCDecode(StrStreamReader(html), true, true);
    if (!elem)
    {
        /* Settle for the raw body instead.
         * TODO: Have the parser be more leinent on errors in HTML mode. */
        char *body = GetRawBody(event);
        Free(html);
        return StrDuplicate(body);
    }

    flags.quote = false;
    xepd = XMPPifyElement(data ? data->config : NULL, event, elem, flags);

    XMLFreeElement(elem);
    Free(html);

    return xepd;
}

char *
ParseeGenerateMTO(char *common_id)
{
    char *matrix_to;
    if (!common_id)
    {
        return NULL;
    }

    /* TODO: Is HttpUrlEncode okay? */
    common_id = HttpUrlEncode(common_id);
    matrix_to = StrConcat(2, "https://matrix.to/#/", common_id);
    Free(common_id);

    return matrix_to;
}
