#include <Parsee.h>

#include <Cytoplasm/Json.h>
#include <Cytoplasm/Util.h>

#include <Glob.h>
#include <AS.h>

void
ParseeGlobalUnban(ParseeData *data, char *glob)
{
    DbRef *ref;
    HashMap *j;
    if (!data || !glob)
    {
        return;
    }

    ref = DbLock(data->db, 1, "global_bans");
    if (!ref)
    {
        ref = DbCreate(data->db, 1, "global_bans");
    }

    j = DbJson(ref);

    JsonValueFree(HashMapDelete(j, glob));

    DbUnlock(data->db, ref);
}
void
ParseeGlobalBan(ParseeData *data, char *glob, char *reason)
{
    DbRef *ref;
    HashMap *j, *obj;
    if (!data || !glob)
    {
        return;
    }

    ref = DbLock(data->db, 1, "global_bans");
    if (!ref)
    {
        ref = DbCreate(data->db, 1, "global_bans");
    }

    j = DbJson(ref);

    obj = HashMapCreate();
    if (reason)
    {
        HashMapSet(obj, "reason", JsonValueString(reason));
    }
    HashMapSet(obj, "date", JsonValueInteger(UtilTsMillis()));
    JsonValueFree(HashMapSet(j, glob, JsonValueObject(obj)));

    DbUnlock(data->db, ref);
}
bool
ParseeManageBan(ParseeData *data, char *user, char *room)
{
    DbRef *ref;
    HashMap *j;
    char *key;
    JsonValue *val;
    bool banned = false , matches = false;
    if (!data || !user)
    {
        return false;
    }

    ref = DbLockIntent(data->db, DB_HINT_READONLY, 1, "global_bans");
    j = DbJson(ref);
    while (HashMapIterate(j, &key, (void **) &val))
    {
        HashMap *obj = JsonValueAsObject(val);
        if (matches)
        {
            continue;
        }
        if (GlobMatches(key, user))
        {
            banned = true;
            matches = true; 
            if (room)
            {
                /* TODO: Use the object to set the reason */
                ASBan(data->config, room, user);
                (void) obj;
            }
        }
    }
    DbUnlock(data->db, ref);
 
    return banned;
}

