#include <MUCServ.h>

#include <Cytoplasm/Memory.h>
#include <Cytoplasm/Str.h>
#include <Cytoplasm/Log.h>

#include <pthread.h>
#include <string.h>

#define MUCNS "http://jabber.org/protocol/muc"

struct MUCServer {
    pthread_mutex_t mut;

    ParseeData *data;
};

static char *
GetMUCName(char *jid)
{
    char *name, *at;
    size_t len;
    if (!jid || *jid != '#')
    {
        return NULL;
    }

    jid++;

    at = strchr(jid, '@');
    if (!at)
    {
        return StrDuplicate(jid);
    }
    len = at - jid;
    name = Malloc(len + 1);
    memset(name, '\0', len + 1);
    memcpy(name, jid, len);

    return name;
}

MUCServer * 
CreateMUCServer(ParseeData *data)
{
    MUCServer *server;
    if (!data)
    {
        return NULL;
    }

    server = Malloc(sizeof(*server));
    pthread_mutex_init(&server->mut, NULL);
    server->data = data;
    return server;
}

static bool
MUCManagePresence(MUCServer *serv, XMLElement *stanza, char *from, char *to)
{
    char *name;
    XMLElement *x = XMLookForTKV(stanza, "x", "xmlns", MUCNS);
    char *type = HashMapGet(stanza->attrs, "type");
    if (x && !type)
    {
        /* The user is trying to join the MUC */
        name = GetMUCName(to);
        Log(LOG_WARNING, "%s is trying to join MUC '%s'", from, name);

        /* TODO: Check if the user should be joining. If so, make them. 
         * Implementing MUCs is gonna be fun. */

        /* TODO: Presence broadcast hell. */
        Free(name);
    }
    return false;
}

static bool
MUCManageMessage(MUCServer *serv, XMLElement *stanza, char *from, char *to)
{
    Log(LOG_WARNING, "MUCSERV: got a message %s->%s", from, to);
    return false;
}

bool 
ManageMUCStanza(MUCServer *serv, XMLElement *stanza)
{
    char *stype, *from, *to;
    bool ret;
    if (!serv || !stanza)
    {
        return false;
    }

    from = HashMapGet(stanza->attrs, "from");
    to = HashMapGet(stanza->attrs, "to");
    stype = stanza->name;

    if (to && *to != '#')
    {
        /* We aren't interacting with a MUC at all. Don't do anything. */
        return false;
    }
    if (StrEquals(stype, "iq"))
    {
        /* TODO: Worry about IQ later */
        return false;
    }

    ret = false;
    pthread_mutex_lock(&serv->mut);
    if (StrEquals(stype, "presence"))
    {
        ret = MUCManagePresence(serv, stanza, from, to);
    }
    else if (StrEquals(stype, "message"))
    {
        ret = MUCManageMessage(serv, stanza, from, to);
    }
    /* TODO: Do stuff while locked */
    pthread_mutex_unlock(&serv->mut);

    /* TODO: Verify the destination, and make sure we aren't doing 
     * anything stupid(especially with discovery) */
    (void) ret;
    return false;
}

bool 
MUCServerExists(MUCServer *serv, char *muc)
{
    if (!serv || !muc)
    {
        return false;
    }

    return false;
}

void 
FreeMUCServer(MUCServer *serv)
{
    if (!serv)
    {
        return;
    }

    pthread_mutex_lock(&serv->mut);
    /* TODO */
    pthread_mutex_unlock(&serv->mut);
    pthread_mutex_destroy(&serv->mut);
    Free(serv);
}
