#include <Cytoplasm/HashMap.h>
#include <Cytoplasm/Memory.h>
#include <Cytoplasm/Json.h>
#include <Cytoplasm/Util.h>
#include <Cytoplasm/Str.h>
#include <Cytoplasm/Log.h>

#include <XMPPFormTool.h>
#include <XMPPCommand.h>
#include <Matrix.h>
#include <Parsee.h>
#include <XMPP.h>
#include <XML.h>
#include <AS.h>

void
MUCUnlink(XMPPCommandManager *m, char *from, XMLElement *form, XMLElement *out)
{
    ParseeData *data = XMPPGetManagerCookie(m);
    char *affiliation, *role;
    char *chat_id;
    char *parsee;
    char *room;
    char *muc;

    ParseeLookupAffiliation(from, &affiliation, &role);
    Free(role);
    if (!StrEquals(affiliation, "owner"))
    {
        SetNote("error", "Unlinking a MUC requires the 'owner' affiliation.");
        Free(affiliation);
        return;
    }

    muc = ParseeTrimJID(from);
    chat_id = ParseeGetFromMUCID(data, muc);
    room = ParseeGetRoomID(data, chat_id);
    if (!chat_id)
    {
        SetNote("error", "Couldn't fetch chat ID.");
        Free(muc);
        return;
    }
    ParseeUnlinkRoom(data, chat_id);

    parsee = ParseeMXID(data);
    Free(ASSend(
        data->config, room, parsee, 
        "m.room.message",
        MatrixCreateNotice("This room has been unlinked."),
        0
    ));
    ASLeave(data->config, room, parsee);

    XMPPLeaveMUC(data->jabber, "parsee", muc, "Unlinked by MUC admin.");

    /* Setting an error here won't work, as we're communicating through 
     * the MUC, which we *left*. I guess we can try to defer the leave. */
    SetNote("info", "Unlinked MUC.");

    Free(affiliation);
    Free(chat_id);
    Free(parsee);
    Free(room);
    Free(muc);
    (void) form;
}
