#include <Cytoplasm/HashMap.h>
#include <Cytoplasm/Memory.h>
#include <Cytoplasm/Json.h>
#include <Cytoplasm/Util.h>
#include <Cytoplasm/Str.h>
#include <Cytoplasm/Log.h>

#include <XMPPFormTool.h>
#include <XMPPCommand.h>
#include <Parsee.h>
#include <XMPP.h>
#include <XML.h>

#define TITLE "Parsee global bans"

void
NoflyCallback(XMPPCommandManager *m, char *from, XMLElement *form, XMLElement *out)
{
    ParseeData *data = XMPPGetManagerCookie(m);
    char *trimmed = ParseeTrimJID(from);
    XMLElement *x;
    XMLElement *title;
    XMLElement *reported, *item, *field, *value, *txt;

    if (!ParseeIsAdmin(data, trimmed))
    {
        SetNote("error", "User is not authorised to execute command.");

        Free(trimmed);
        return;
    }

    x = XMLCreateTag("x");
    XMLAddAttr(x, "xmlns", "jabber:x:data");
    title = XMLCreateTag("title");
    XMLAddChild(x, title);
    XMLAddChild(out, x);

    Free(trimmed);

    SetTitle(x, TITLE);

    XMLAddAttr(x, "type", "result");
    {
        DbRef *ref = DbLock(data->db, 1, "global_bans");
        HashMap *bans;
        char *banned, *reason;
        JsonValue *ban_val;
        reported = XMLCreateTag("reported");
        XMLAddChild(x, reported);

        if (!ref)
        {
            ref = DbCreate(data->db, 1, "global_bans");
        }
        
        bans = DbJson(ref);

        /* Report */
        Report("ban", "Banned glob");
        Report("reason", "Reason for the nofly");
        Report("ts", "UNIX timestamp for creation");

        /* Set */
        while (HashMapIterate(bans, &banned, (void **) &ban_val))
        {
            HashMap *ban_obj = JsonValueAsObject(ban_val);
            reason = GrabString(ban_obj, 1, "reason");
            char *ts = StrDuplicate("N/A");
            if (!reason)
            {
                reason = "N/A";
            }
            if (HashMapGet(ban_obj, "date"))
            {
                Free(ts);
                ts = StrInt(GrabInteger(ban_obj, 1, "date") / (1 SECONDS));
            }

            BeginItem();
            SetField("ban", banned);
            SetField("reason", reason);
            SetField("ts", ts);
            EndItem();

            Free(ts);
        }
        DbUnlock(data->db, ref);
    }

    (void) form;
}
