#include <Cytoplasm/Memory.h>
#include <Cytoplasm/Base64.h>
#include <Cytoplasm/Util.h>
#include <Cytoplasm/Str.h>
#include <Cytoplasm/Sha.h>
#include <Cytoplasm/Log.h>

#include <StringStream.h>
#include <XMPPCommand.h>
#include <Matrix.h>
#include <XMPP.h>
#include <XML.h>

#include "XMPPThread/internal.h"

IQFeatures * 
CreateIQFeatures(void)
{
    IQFeatures *ret = Malloc(sizeof(*ret));

    ret->identity = ArrayCreate();
    ret->adverts = ArrayCreate();

    return ret;
}
void 
FreeIQFeatures(IQFeatures *features)
{
    size_t i;
    if (!features)
    {
        return;
    }

    for (i = 0; i < ArraySize(features->adverts); i++)
    {
        Free(ArrayGet(features->adverts, i));
    }
    ArrayFree(features->adverts);

    for (i = 0; i < ArraySize(features->identity); i++)
    {
        XMPPIdentity *identity = ArrayGet(features->identity, i);

        Free(identity->category);
        Free(identity->type);
        Free(identity->lang);
        Free(identity->name);

        Free(identity);
    }
    ArrayFree(features->identity);

    Free(features);
}

void 
AdvertiseIQFeature(IQFeatures *f, char *feature)
{
    if (!f || !feature)
    {
        return;
    }

    ArrayAdd(f->adverts, StrDuplicate(feature));
}
void 
AddIQIdentity(IQFeatures *f, char *cat, char *lang, char *type, char *name)
{
    XMPPIdentity *identity;
    if (!f)
    {
        return;
    }

    identity = Malloc(sizeof(*identity));
    identity->category = StrDuplicate(cat);
    identity->type = StrDuplicate(type);
    identity->lang = StrDuplicate(lang);
    identity->name = StrDuplicate(name);
    ArrayAdd(f->identity, identity);
}
/* Generates a SHA-256 hash of the ver field. */
char *
XMPPGenerateVer(IQFeatures *features)
{
    char *S = NULL;
    unsigned char *Sha = NULL;
    size_t i;

    /* Initialise identity table, to be sorted */
    ArraySort(features->identity, IdentitySort);
    for (i = 0; i < ArraySize(features->identity); i++)
    {
        XMPPIdentity *identity = ArrayGet(features->identity, i);
        char *id_chunk = StrConcat(7, 
            identity->category, "/",
            identity->type, "/",
            identity->lang, "/",
            identity->name);
        char *tmp = S;
        S = StrConcat(3, S, id_chunk, "<");
        Free(tmp);
        Free(id_chunk);
    }

    ArraySort(features->adverts, ((int (*) (void *, void *)) ICollate));
    for (i = 0; i < ArraySize(features->adverts); i++)
    {
        char *feature = ArrayGet(features->adverts, i);
        char *tmp = S;
        S = StrConcat(3, S, feature, "<");
        Free(tmp);
    }

    Sha = Sha1(S);
    Free(S);
    S = Base64Encode((const char *) Sha, 20);
    Free(Sha);

    return S;
}


void
XMPPAnnotatePresence(XMLElement *presence, IQFeatures *features)
{
    XMLElement *c;
    char *ver;
    if (!presence || !features)
    {
        return;
    }

    ver = XMPPGenerateVer(features);
    c = XMLCreateTag("c");
    XMLAddAttr(c, "xmlns", "http://jabber.org/protocol/caps");
    XMLAddAttr(c, "hash", "sha-1");
    XMLAddAttr(c, "node", REPOSITORY);
    XMLAddAttr(c, "ver", ver);

    Free(ver);
    XMLAddChild(presence, c);
}


IQFeatures * 
LookupJIDFeatures(char *jid)
{
    IQFeatures *features;
    if (!jid)
    {
        return NULL;
    }

    features = CreateIQFeatures();

    if (*jid == '#')
    {
        /* This is a MUC. As such, we need to advertise MUCs */
#define ID(...) AddIQIdentity(features, __VA_ARGS__)
#define AD(var) AdvertiseIQFeature(features, var)
        ID("gateway", NULL, "matrix", "Parsee MUC gateway");
        ID("conference", NULL, "text", "Parsee MUC gateway");
        ID("component", NULL, "generic", "Parsee component");

        AD("http://jabber.org/protocol/muc");
#undef AD
#undef ID
    }
    else
    {
#define IdentitySimple(cat, Type, Name) AddIQIdentity(features, cat, NULL, Type, Name);
    IQ_IDENTITY
#undef IdentitySimple
#define AdvertiseSimple(feature) AdvertiseIQFeature(features, feature);
    IQ_ADVERT
#undef AdvertiseSimple
    }
    return features;
}
