#ifndef PARSEE_MUCSERV_H
#define PARSEE_MUCSERV_H

/*-* An easy interface for handling MUCs.
 */

typedef struct MUCServer MUCServer;

#include <Parsee.h>

#include <stdbool.h>

/** Creates a MUC server handle given a ParseeData handle. 
 * This handle shall be used for all MUC-related operations.
 * ----------------------------------------------------------
 * Returns: a MUC server handle[HEAP] | NULL
 * Thrasher: FreeMUCServer
 * See-Also: https://xmpp.org/extensions/xep-0045.html */
extern MUCServer * CreateMUCServer(ParseeData *data);

/** Manages a stanza from a MUC, and returns true if the stanza 
 * was actually processed by the MUC server.
 * -------------------------------------------------------------
 * See-Also: CreateMUCServer */
extern bool ManageMUCStanza(MUCServer *serv, XMLElement *stanza);

/** Checks if a MUC(from its localpart, so 
 * <code>'#foobar@bridge.blow.hole' -> 'foobar'</code>) exists.
 * --------------
 * Returns: whenever the MUC exists */
extern bool MUCServerExists(MUCServer *serv, char *muc);

/** Destroys all memory and references from a MUC server handle.
 * --------------
 * Thrashes: CreateMUCServer
 * See-Also: https://xmpp.org/extensions/xep-0045.html */
extern void FreeMUCServer(MUCServer *serv);

#endif
